import requests
import time
from datetime import datetime
import json
import numpy as np
import schedule


# Creazione classe Bot con funzione per accedere ai dati di coinmarketcap.com
class Bot:
    def __init__(self):
        self.url = 'https://pro-api.coinmarketcap.com/v1/cryptocurrency/listings/latest'
        self.params = {
            'start': '1',
            'limit': '100',
            'convert': 'USD'
            }
        self.headers = {
            'Accepts': 'application/json',
            'X-CMC_PRO_API_KEY': 'c5d5a545-ec96-4f82-8dbb-200f7f97ddd9'
            }
        self.orders = []
    
    def fetchCurrenciesData(self):
        r = requests.get(url=self.url, headers=self.headers, params=self.params).json()
        return r['data']

reportBot = Bot()

def cryptocurrencies_report(): #funzione che che genera il report in formato json
    r = reportBot.fetchCurrenciesData()
    results['report_time_stamp'] = str(datetime.now().date())+" "+str(datetime.now().time()).split('.')[0]
    results = {}    #dizionario che contiene i dati richiesti
    cryptos = []    #lista che viene aggiornata con i dati richiesti
    
    #ricerca della criptovaluta con il volume più alto nelle ultime 24h
    for i in r:
            cryptos.append((i['name'], i['quote']['USD']['volume_24h'])) #lista di tuple nome-valore 
    
    #ordiniamo la lista in ordine decrescente       
    cryptos.sort(key=lambda x: x[1], reverse=True)
    
    #visto l'ordine di grandezza, scriviamo il volume in notazione esponenziale
    max_volume_cc = "{:.2e}".format(cryptos[0][1])
    
    #aggiugniamo il nome e il valore del volume al dizionario
    results['max_volume_24h_CC'] = cryptos[0][0]+": "+str(max_volume_cc)
    cryptos = []    #svuotiamo la lista
    #ricerca delle migliori e peggiori 10 cripto per incremento %
    for i in r:
            cryptos.append((i['name'], round(i['quote']['USD']['percent_change_24h'], 2)))
    #ordiniamo la lista in base all'incremento %        
    cryptos.sort(key=lambda x: x[1], reverse=True)  
    #creiamo una lista di stringhe con nome della criptovaluta e incremento % e la aggiungiamo al dizionario
    results['10_CCs_best_%_change24h'] = [cryptos[i][0]+": "+str(cryptos[i][1])+"%" for i in range(10)]
    #invertiamo l'ordine della lista per ottenere facilmente le ultime dieci per incremento %
    cryptos.sort(key=lambda x: x[1])
    results['10_CCs_worst_%_change24h'] = [cryptos[i][0]+": "+str(cryptos[i][1])+"%" for i in range(10)]
    
    #creiamo una lista delle prime 20 criptovalute della classifica di cmc
    best_ccs = [i['name'] for i in r][:20]
    money_for_20cc = 0 #contatore
    
    #in base ai nomi presenti nella lista andiamo a cercare il relativo prezzo e lo aggiungiamo a un contatore
    for i in best_ccs:
        for j in r:
            if i == j['name']:
                money_for_20cc += j['quote']['USD']['price']
                
    #aggiungiamo al dizionario gli usd necessari per acquistare un'unità delle prime 20 criptovalute        
    results['usd_for_1st_20_CCs_by_mcap'] = round(money_for_20cc, 2)
    
    cryptos = []    #svuotiamo la lista
    
    money_for_cc_vol = 0    #contatore
    
    #cerchiamo le criptovalute che hanno avuto un volume maggiore di 76000000 nelle ultime 24h
    for i in r:
        if i['quote']['USD']['volume_24h'] > 76000000:
            cryptos.append(i['name'])
    
    #cerchiamo il relativo prezzo delle criptovaulte trovate e incrementiamo un contatore
    for i in cryptos:
        for j in r:
            if i == j['name']:
                money_for_cc_vol += j['quote']['USD']['price']
    
    #aggiungiamo al dizionario gli usd necessari per acquistare un'unità delle criptovalute trovate          
    results['usd_for_high_volumes_24h_CCs'] = round(money_for_cc_vol, 2)
    
    cryptos = []    #svuotiamo la lista
    
    #aggiungiamo alla lista delle tuple con nome, prezzo e incremento % 24h delle prime 20 criptovalute
    for i in r:
            cryptos.append((i['name'], i['quote']['USD']['percent_change_24h'], i['quote']['USD']['price']))
    cryptos = cryptos[:20]
    #calcoliamo quanto avremmo pagato ieri le criptovalute
    yesterday_total_investment = np.array([i[2]/(1+i[1]/100) for i in cryptos]).sum()
    #calcoliamo il rendiemento dell'investimento sulla base dei prezzi di oggi
    return_for_investment_20_best_ccs = ((money_for_20cc - yesterday_total_investment) / yesterday_total_investment)*100
    #aggiungiamo al dizionario il rendimento 
    results['daily_%_return_on_1st_20_CCs_by_mcap'] = round(return_for_investment_20_best_ccs, 2)
    #creiamo il nome del file in modo che ogni giorno corrisponda alla data del giorno stesso
    today = datetime.today()
    file_name_save = (str(today).split()[0] + '.json').replace('-','_')
    #salviamo il file in formato json e lo rendiamo più leggibile con separatori e indentazione
    with open(file_name_save, 'w') as fp:
        json.dump(results,fp,indent=1,separators=("\n",":\n\n"))
        
#con la libreria schedule impostiamo la generazione del report alle 16:00 di ogni giorno
schedule.every().day.at("16:00").do(cryptocurrencies_report)

#utilizziamo un ciclo infinito per far girare lo script ogni giorno
while True:
    schedule.run_pending()
    time.sleep(1)
    


